/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.webkit.WebView;
import android.widget.Button;
import android.widget.Toast;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;

import mdbdev.es.droidscrape.model.DroidScrapeResultsReport;
import mdbdev.es.droidscrape.utils.files.FileInterface;

/**
 * Activity that shows the results of a plugin inside a WebView
 */
public class DroidScrapeResultActivity extends AppCompatActivity {

    private String data;
    private String loc;
    private String filename;
    private File dir;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_result);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);

        Intent intent = getIntent();
        DroidScrapeResultsReport result = intent.getParcelableExtra(getString(R.string.result_object_name));
        data = result.getHTMLReport();
        //Set up button listeners
        Button btShare = (Button) findViewById(R.id.share);
        btShare.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    sendEmail();
                } catch (IOException e) {
                    e.printStackTrace();
                    emailError(e.getMessage());
                }
            }
        });

        if(data != null) {
            fillWebView();
            try {
                generateReportFile();
            } catch (IOException e) {
                e.printStackTrace();
                Toast.makeText(this, R.string.error_cant_create_file,Toast.LENGTH_LONG).show();
            }
        }
    }

    /**
     * Generates an html file that contains the full report information
     * @throws IOException
     */
    private void generateReportFile() throws IOException {
        //Generate file with the data
        File sdCard = Environment.getExternalStorageDirectory();
        dir = new File(sdCard.getAbsolutePath() + getString(R.string.reportsLocation));
        dir.mkdirs();
        filename = getString(R.string.report_file_name);
        File file = new File(dir, filename);

        FileOutputStream f = new FileOutputStream(file);
        Writer writer = new BufferedWriter(new OutputStreamWriter(f,"utf-8"));
        writer.write(data);
        writer.close();

        loc = "file://"+ dir.getAbsolutePath()+"/"+ filename;

        file.deleteOnExit();
    }

    private void emailError(String message) {
        Toast.makeText(this,"Error sending email: "+message,Toast.LENGTH_LONG).show();
    }

    /**
     * Sends the HTML information to a mail client
     */
    private void sendEmail() throws IOException {
        if(loc != null) {
            final Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);

            emailIntent.setType("message/rfc822");
            emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, getString(R.string.email_subject));
            emailIntent.putExtra(android.content.Intent.EXTRA_TEXT, getString(R.string.email_body));
            emailIntent.putExtra(Intent.EXTRA_STREAM, Uri.parse(loc));
            startActivity(Intent.createChooser(emailIntent, getString(R.string.send_email)));
        }
    }

    /**
     * Fills the webview with the results received
     */
    protected void fillWebView() {
        WebView webview = (WebView)this.findViewById(R.id.resultsView);
        webview.loadDataWithBaseURL("", data, "text/html", "UTF-8", "");
    }

    /**
     * Returning finishes this activity and clears it from the stack.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            // Respond to the action bar's Up/Home button
            case android.R.id.home:
                returnHome();
                break;
            case R.id.action_home:
                returnHome();
                break;
            case R.id.action_clean:
                clearFiles();
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    /**
     * Forcefully deletes all the files generated by the app
     */
    /**
     * Forcefully deletes all the files generated by the app
     */
    private void clearFiles() {
        FileInterface.deleteFiles(this);
        showToast(getString(R.string.files_deleted));
    }

    /**
     * Returns to the main menu
     */
    private void returnHome() {
        Intent homeIntent = new Intent(this, MainMenuActivity.class);
        homeIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivity(homeIntent);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main_menu, menu);
        return true;
    }
    /**
     * Shows a toast with the specified message
     * @param msg Message to show
     */
    protected void showToast(String msg) {
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_LONG).show();
    }


}
