/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape;

import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.View;
import android.view.Menu;
import android.view.MenuItem;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.Toast;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.LinkedList;
import java.util.List;

import mdbdev.es.droidscrape.plugins.DroidScrapePlugin;
import mdbdev.es.droidscrape.plugins.DroidScrapePluginManager;

/**
 * Main activity of the App featuring a menu with all the available plugins
 */
public class MainMenuActivity extends AppCompatActivity {
    private DroidScrapePluginManager pmanager;
    private List<DroidScrapePlugin> plugins;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main_menu);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        //Setup feedback button
        FloatingActionButton feedback = (FloatingActionButton)findViewById(R.id.fab);
        feedback.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                feedbackRequest();
            }
        });

        // Plugin loading
        plugins = new LinkedList<>();
        pmanager = new DroidScrapePluginManager(this.getBaseContext());
        try {
            plugins = pmanager.loadPlugins();
            generateButtons();
        } catch (IllegalAccessException | IOException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            badConfigException();
            e.printStackTrace();
        } catch (InstantiationException e) {
            badConfigException();
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            badConfigException();
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            badConfigException();
            e.printStackTrace();
        }
    }

    /**
     * Generates a feedback request to be launched by the users email app
     */
    private void feedbackRequest() {
        final Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);

        emailIntent.setType("message/rfc822");
        emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, getString(R.string.feedback_email_subject));
        emailIntent.putExtra(Intent.EXTRA_EMAIL,new String[]{getString(R.string.contact_email)});
        startActivity(Intent.createChooser(emailIntent, getString(R.string.send_feedback_email)));
    }

    /**
     * Logs an error in plugin configuration files
     */
    private void badConfigException(){
        Log.i(getString(R.string.plugin_config_error_tag),getResources().getString(R.string.pluginConfigError));
        Toast.makeText(getApplicationContext(), getResources().getString(R.string.pluginConfigError), Toast.LENGTH_LONG).show();
    }

    @Override
    public void onStart(){
        super.onStart();
    }

    /**
     * Generate the plugin buttons and sets their event listeners
     */
    private void generateButtons() {
        //Add plugin buttons
        ViewGroup mainContent = (ViewGroup) findViewById(R.id.main_content);
        for(final DroidScrapePlugin p : plugins){
            Button bt = new Button(this);
            bt.setText(p.getPluginInformation().getPluginName());
            int iconId = getApplicationContext().getResources().getIdentifier(p.getPluginInformation().getPluginIconUrl(), "drawable", getApplicationContext().getPackageName());
            Drawable dr = getResources().getDrawable(iconId);
            Bitmap bitmap = ((BitmapDrawable) dr).getBitmap();
            // Scale it to 50 x 50
            Drawable d = new BitmapDrawable(getResources(), Bitmap.createScaledBitmap(bitmap, 50, 50, true));
            bt.setCompoundDrawablesWithIntrinsicBounds(d,null,null,null);
            bt.setLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                    ViewGroup.LayoutParams.WRAP_CONTENT));

            bt.setOnClickListener(new View.OnClickListener() {
                public void onClick(View v) {
                    p.start();
                }
            });


            mainContent.addView(bt);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        //getMenuInflater().inflate(R.menu.menu_main_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        /*if (id == R.id.action_settings) {
            return true;
        }*/
        return super.onOptionsItemSelected(item);
    }
}
