/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape.plugins;

import android.content.Context;

import com.google.gson.Gson;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import mdbdev.es.droidscrape.R;
import mdbdev.es.droidscrape.model.PluginInformation;

/**
 * Created by Mario on 25/03/2017.
 * Plugin Manager class
 */

public class DroidScrapePluginManager {
    private Context context;
    public DroidScrapePluginManager(Context context){
        this.context = context;
    }
    /**
     * Loads all the installed plugins into a List reading their info from Json files inside the raw resources folder
     * @return DroidScrapePlugin List with all the installed plugins
     */
    public List<DroidScrapePlugin> loadPlugins() throws IllegalAccessException, IOException, ClassNotFoundException, NoSuchMethodException, InvocationTargetException, InstantiationException {
        List<DroidScrapePlugin> plugins = new LinkedList<>();
        List<Integer> resIds = listRaw();
        for(Integer resId : resIds){
            String json = readJSONString(resId);
            Map jsonObject = new Gson().fromJson(json, Map.class);
            String pluginName = (String) jsonObject.get(context.getResources().getString(R.string.pluginNameField));
            String pluginClass = (String) jsonObject.get(context.getResources().getString(R.string.pluginClassField));
            String pluginIcon = (String) jsonObject.get(context.getResources().getString(R.string.pluginIconField));

            PluginInformation pinfo = new PluginInformation(pluginName,pluginClass,pluginIcon);
            Class<?> classObj = Class.forName(pluginClass);
            Constructor<?> constructor = classObj.getConstructor(Context.class, PluginInformation.class);
            DroidScrapePlugin instance = (DroidScrapePlugin) constructor.newInstance(context, pinfo);

            plugins.add(instance);
        }
        return plugins;
    }

    /**
     * Reads a JSON resource into a String
     * @param resId Int ID of the resource
     * @return String with the JSON in String format
     * @throws IOException
     */
    private String readJSONString(Integer resId) throws IOException {
        InputStream is = context.getResources().openRawResource(resId);
        Writer writer = new StringWriter();
        char[] buffer = new char[1024];

        Reader reader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
        int n;
        while ((n = reader.read(buffer)) != -1) {
            writer.write(buffer, 0, n);
        }
        is.close();

        return writer.toString();
    }

    /**
     * Lists the resources existing in the raw folder
     * @return List with the int ID of al the resources in raw folder
     * @throws IllegalAccessException
     */
    private List<Integer> listRaw() throws IllegalAccessException {
        List<Integer> resIds = new LinkedList<>();
        Field[] fields=R.raw.class.getFields();
        for(Field f : fields){
            int resourceID=f.getInt(f);
            resIds.add(resourceID);
        }
        return resIds;
    }
}
