/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape.plugins.emailheaderanalyzer;

import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import mdbdev.es.droidscrape.MainMenuActivity;
import mdbdev.es.droidscrape.R;
import mdbdev.es.droidscrape.plugins.PluginMainActivity;
import mdbdev.es.droidscrape.utils.files.FileInterface;

/**
 * Main activity of the Email Header Analyzer plugin
 */
public class EmailHeaderAnalyzerMainActivity extends PluginMainActivity {
    private EditText emailHeaderContents;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_email_header_analyzer_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);

        emailHeaderContents = (EditText) findViewById(R.id.emailHeaderContents);

        //Manage paste button
        final Button pasteBt = (Button) findViewById(R.id.paste);
        pasteBt.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v) {
                ClipboardManager clipboard = (ClipboardManager) getSystemService(Context.CLIPBOARD_SERVICE);
                String pasteData = "";

                if(clipboard.hasPrimaryClip()){
                    ClipData.Item clipItem = clipboard.getPrimaryClip().getItemAt(0);
                    pasteData = clipItem.getText().toString();
                    emailHeaderContents.setText(pasteData);
                }
            }
        });

        //Manage analyze button
        final Button analyzeBt = (Button) findViewById(R.id.analyze);
        analyzeBt.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v) {
                Log.i("DEBUG","TEXT: "+emailHeaderContents.getText());
                if(emailHeaderContents.getText().toString().equals("")){
                    showToast(getApplicationContext().getResources().getString(R.string.header_field_empty_error));
                }else{
                    startProcessActivity();
                }
            }
        });
    }

    /**
     * Starts the processing activity
     */
    private void startProcessActivity() {
        Intent intent = new Intent(this, EmailHeaderAnalyzerProcessActivity.class);
        intent.putExtra("headerString", emailHeaderContents.getText().toString());
        startActivity(intent);
    }

}
