/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape.plugins.emailheaderanalyzer;

import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import java.io.UnsupportedEncodingException;

import javax.mail.MessagingException;

import mdbdev.es.droidscrape.MainMenuActivity;
import mdbdev.es.droidscrape.R;
import mdbdev.es.droidscrape.DroidScrapeResultActivity;
import mdbdev.es.droidscrape.plugins.emailheaderanalyzer.analyzer.EmailHeaderAnalysisListener;
import mdbdev.es.droidscrape.plugins.emailheaderanalyzer.analyzer.EmailHeaderAnalyzer;
import mdbdev.es.droidscrape.plugins.emailheaderanalyzer.analyzer.model.EmailHeadersReport;

/**
 * Activity that shows the progress of the Email Header Analyzer analysis
 */
public class EmailHeaderAnalyzerProcessActivity extends AppCompatActivity {

    private AsyncTask<String, String, EmailHeadersReport> analyzeTask;
    private TextView statusView;

    /**
     * Asynchronous task that performs the analysis
     */
    public class AnalyzeHeaderTask extends AsyncTask<String, String, EmailHeadersReport> implements EmailHeaderAnalysisListener {

        @Override
        protected EmailHeadersReport doInBackground(String... strings) {
            EmailHeadersReport result;
            EmailHeaderAnalyzer analyzer = new EmailHeaderAnalyzer();
            analyzer.addListener(this);
            try {
                result = analyzer.analyze(strings[0]);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                result = null;
            } catch (MessagingException e) {
                e.printStackTrace();
                result = null;
            }

            return result;
        }

        @Override
        protected void onProgressUpdate(String... values) {
            updateStatus(values[0]);
        }

        @Override
        protected void onPostExecute(EmailHeadersReport result){
            showResultsActivity(result);
        }

        public void updateProgress(String s) {
            publishProgress(s);
        }
    }

    /**
     * Updates status text
     * @param s Status text
     */
    public void updateStatus(String s){
        this.statusView.setText(s);
    }

    /**
     * Shows the results in the Results Activity
     * @param result
     */
    private void showResultsActivity(EmailHeadersReport result) {
        Intent intent = new Intent(this, DroidScrapeResultActivity.class);
        intent.putExtra(getString(R.string.result_object_name), result);
        startActivity(intent);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_email_header_analyzer_process);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        Intent intent = getIntent();
        String headerString = intent.getExtras().getString("headerString");

        analyzeTask = new AnalyzeHeaderTask().execute(headerString);
        // Set up the cancel button
        Button btCancel = (Button) findViewById(R.id.cancel);
        btCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                analyzeTask.cancel(true);
                if(analyzeTask.isCancelled())
                    returnHome();
            }
        });

        statusView = (TextView) findViewById(R.id.statusView);
    }

    /**
     * Returns to the main menu
     */
    private void returnHome() {
        Intent homeIntent = new Intent(this, MainMenuActivity.class);
        homeIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivity(homeIntent);
    }

}
