/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape.plugins.emailheaderanalyzer.analyzer.ipapi;

import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Date;

/**
 * Created by Mario on 26/04/2017.
 * Class to perform requests to the ip-api.com geolocation API
 */
public class IpApi {

    public static int REQUEST_COUNT = 0;
    public static long START_TIMESTAMP = 0;

    private static long MAX_MILLISECONDS = 60000;
    private static int MAX_REQUEST_PER_MINUTE = 120;

    /**
     * Queries IP API for an IP address. Checks if the app is close to the limit established by the API service
     * @param ipAddress IP to query for
     * @return JSONObject containing the API response
     * @throws IOException
     * @throws JSONException
     */
    public static JSONObject queryForIp(String ipAddress) throws IOException, JSONException {
        long currentTimestamp = new Date().getTime();
        long elapsed = currentTimestamp-START_TIMESTAMP;
        if(REQUEST_COUNT > MAX_REQUEST_PER_MINUTE && elapsed<MAX_MILLISECONDS){
            try {
                Thread.sleep(60000);
                REQUEST_COUNT = 0;
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        JSONObject ipApi = getJSONObjectFromURL("http://ip-api.com/json/"+ipAddress);
        REQUEST_COUNT++;
        return ipApi;
    }

    /**
     * Gets a JSONObject from a given URL
     * @param urlString URL of the service
     * @return JSONObject containing the service response
     * @throws IOException
     * @throws JSONException
     */
    private static JSONObject getJSONObjectFromURL(String urlString) throws IOException, JSONException {

        HttpURLConnection urlConnection = null;

        URL url = new URL(urlString);

        urlConnection = (HttpURLConnection) url.openConnection();

        urlConnection.setRequestMethod("GET");
        urlConnection.setReadTimeout(10000 /* milliseconds */);
        urlConnection.setConnectTimeout(15000 /* milliseconds */);

        urlConnection.setDoOutput(true);

        urlConnection.connect();

        BufferedReader br=new BufferedReader(new InputStreamReader(url.openStream()));

        char[] buffer = new char[1024];

        String jsonString = new String();

        StringBuilder sb = new StringBuilder();
        String line;
        while ((line = br.readLine()) != null) {
            sb.append(line+"\n");
        }
        br.close();

        jsonString = sb.toString();

        return new JSONObject(jsonString);
    }
}
