/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape.plugins.emailheaderanalyzer.analyzer.model;

import android.os.Parcel;

import java.util.ArrayList;
import java.util.List;

import mdbdev.es.droidscrape.model.DroidScrapeResultsReport;

/**
 * Created by Mario on 04/05/2017.
 */

public class EmailHeadersReport implements DroidScrapeResultsReport{
    private List<AnalyzedHeader> headers;

    public EmailHeadersReport(List<AnalyzedHeader> headers) {
        this.headers = headers;
    }
    public EmailHeadersReport(){
        this.headers = new ArrayList<>();
    }

    public void addHeader(AnalyzedHeader h){
        this.headers.add(h);
    }

    public EmailHeadersReport(Parcel in){
        this.headers = new ArrayList<>();
        readFromParcel(in);
    }

    private void readFromParcel(Parcel in) {
        in.readTypedList(headers,AnalyzedHeader.CREATOR);
    }

    @Override
    public String getHTMLReport() {
        String result = "<h1>Email header analysis</h1>";
        result +="<table style=\"margin:0;width:100%\">";
        for(AnalyzedHeader h : headers){
            result +="<tr><td style=\"font-weight:bold;background-color:#B6D1E6;\">"+h.getName()+"</td><td>"+h.getValue()+"</td></tr><tr><td colspan=\"2\" style=\"background-color:#E4EFF9\">"+h.getAnalysis()+"</td></tr>";
        }
        result +="</table>";
        return result;
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeTypedList(headers);
    }

    public static final Creator<EmailHeadersReport> CREATOR = new Creator<EmailHeadersReport>() {
        @Override
        public EmailHeadersReport createFromParcel(Parcel in) {
            return new EmailHeadersReport(in);
        }

        @Override
        public EmailHeadersReport[] newArray(int size) {
            return new EmailHeadersReport[size];
        }
    };

    public List<AnalyzedHeader> getHeaders() {
        return headers;
    }

    public void setHeaders(List<AnalyzedHeader> headers) {
        this.headers = headers;
    }
}
