/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape.plugins.webscraper;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Patterns;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import mdbdev.es.droidscrape.MainMenuActivity;
import mdbdev.es.droidscrape.R;
import mdbdev.es.droidscrape.plugins.PluginMainActivity;
import mdbdev.es.droidscrape.plugins.emailheaderanalyzer.EmailHeaderAnalyzerProcessActivity;

import static android.R.attr.duration;

/**
 * Main Activity of the Web Scraper plugin
 */
public class WebScraperMainActivity extends PluginMainActivity {

    private EditText txtUrl;
    private Spinner spinDepthLevel;
    private EditText txtSearchTerms;
    private Spinner spinScanMode;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_web_scraper_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);

        // Set up Buttons and listeners
        Button btClear = (Button) findViewById(R.id.btClearForm);
        Button btStart = (Button) findViewById(R.id.btStartScraper);
        btClear.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                clearForm();
            }
        });
        btStart.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                startScan();
            }
        });

        txtUrl = (EditText) findViewById(R.id.txtUrl);
        spinDepthLevel = (Spinner) findViewById(R.id.spinScanDepth);
        txtSearchTerms = (EditText) findViewById(R.id.txtSearchTerms);
        spinScanMode = (Spinner) findViewById(R.id.spinScanMode);

        clearForm();

        //Validate that input is a valid URL
        txtUrl.addTextChangedListener(new TextWatcher(){
            public void afterTextChanged(Editable s) {}

            public void beforeTextChanged(CharSequence s, int start, int count, int after){}

            public void onTextChanged(CharSequence s, int start, int before, int count){
                String strEnteredVal = txtUrl.getText().toString();

                if(!strEnteredVal.equals("")){
                    Pattern pattern = Patterns.WEB_URL;
                    Matcher matcher = pattern.matcher(strEnteredVal);
                    boolean valid = matcher.find();

                    if(!valid){
                        txtUrl.setError(getString(R.string.error_invalid_url));
                    }else{
                        txtUrl.setError(null);
                    }
                }
            }
        });

        //Warn about aggressive scan mode
        spinScanMode.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                if(id==2){
                    Toast.makeText(parent.getContext(), getString(R.string.warning_aggressive_scan), Toast.LENGTH_LONG).show();
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
    }

    /**
     * Starts the scan if possible
     */
    private void startScan() {
        if(validateFields()){
            Intent intent = new Intent(this, WebScraperProcessActivity.class);
            intent.putExtra("url", txtUrl.getText().toString());
            intent.putExtra("depth", spinDepthLevel.getSelectedItemPosition());
            intent.putExtra("mode", spinScanMode.getSelectedItemPosition());
            intent.putExtra("terms", txtSearchTerms.getText().toString());
            startActivity(intent);
        }else{
            Toast.makeText(this, R.string.error_form_not_validated, Toast.LENGTH_LONG).show();
        }
    }

    /**
     * Fields validation
     * @return true if the fields have valid values
     */
    private boolean validateFields() {
        if(txtUrl.getText().toString().equals("")) txtUrl.setError(getString(R.string.error_field_empty));
        if(txtSearchTerms.getText().toString().equals("")) txtSearchTerms.setError(getString(R.string.error_field_empty));

        return (txtUrl.getError()==null &&
                txtSearchTerms.getError()==null
        );
    }

    /**
     * Clears the whole form
     */
    private void clearForm() {
        txtUrl.setText("");
        txtUrl.setError(null);

        spinDepthLevel.setSelection(1,true);
        spinScanMode.setSelection(0,true);

        txtSearchTerms.setText("");
    }


}
