/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape.plugins.webscraper;

import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import java.net.MalformedURLException;

import mdbdev.es.droidscrape.DroidScrapeResultActivity;
import mdbdev.es.droidscrape.MainMenuActivity;
import mdbdev.es.droidscrape.R;
import mdbdev.es.droidscrape.plugins.webscraper.scraper.model.ScraperMode;
import mdbdev.es.droidscrape.plugins.webscraper.scraper.WebScraper;
import mdbdev.es.droidscrape.plugins.webscraper.scraper.ScraperStatusListener;
import mdbdev.es.droidscrape.plugins.webscraper.scraper.model.ScraperReport;

/**
 * Activity that shows the progress of the Web Scraper plugin scan
 */
public class WebScraperProcessActivity extends AppCompatActivity {

    private TextView txtProgress;
    private TextView txtCurrentUrl;
    private TextView txtLastScanned;
    private TextView txtStatus;
    private ProgressBar progressBar;

    private String url;
    private String terms;
    private int sDepth;
    private int sMode;
    private AsyncTask<String, String, ScraperReport> scrapeTask;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_web_scraper_process);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        Intent intent = getIntent();
        url = intent.getExtras().getString("url");
        if(!url.startsWith("http"))
            url="http://"+url;
        terms = intent.getExtras().getString("terms");
        sDepth = intent.getExtras().getInt("depth");
        sMode = intent.getExtras().getInt("mode");

        scrapeTask = new WebScraperTask().execute();

        Button btCancel = (Button) findViewById(R.id.cancel);
        btCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                scrapeTask.cancel(true);
                if(scrapeTask.isCancelled())
                    returnHome();
            }
        });

        txtProgress = (TextView) findViewById(R.id.txtProgress);
        txtCurrentUrl = (TextView) findViewById(R.id.txtCurrentUrl);
        txtLastScanned = (TextView) findViewById(R.id.txtLastUrl);
        txtStatus = (TextView) findViewById(R.id.txtStatus);
        progressBar = (ProgressBar) findViewById(R.id.progressBar);
        progressBar.setMax(100);
        progressBar.setProgress(0);
    }

    /**
     * Asynchronous task that performs the scan
     */
    public class WebScraperTask extends AsyncTask<String, String, ScraperReport> implements ScraperStatusListener{
        private int linksScanned;
        private int linksPending;
        private String currentUrl;
        @Override
        protected ScraperReport doInBackground(String... strings) {
            linksScanned = 0;
            linksPending = 0;
            currentUrl = "";

            ScraperReport result;

            WebScraper ws = null;
            try {
                ws = new WebScraper(url,terms,sDepth, ScraperMode.fromIndex(sMode));
                ws.addStatusListener(this);
                ws.startScraper();
                ScraperReport rep = ws.getReport();
                result = rep;
                //result = ws.getReport().getHTMLReport();
            } catch (MalformedURLException e) {
                e.printStackTrace();
                result = null;
                //result="<h1>Invalid URL.</h1><p>The URL you provided does not seem to be valid.</p>";
            }

            return result;
        }
        @Override
        protected void onProgressUpdate(String... values) {
            switch(values.length) {
                case 1:
                    updateProgress(getProgressString(),getProgress());
                    updateCurrentUrl(values[0]);
                    break;
                case 2:
                    updateProgress(getProgressString(),getProgress());
                    updateLastScanned(values[1]);
                    break;
                case 3:
                    updateProgress(getProgressString(),getProgress());

                    break;
                case 4:
                    updateWaitingStatus(values[3]);
                    break;
                case 5:
                    alertUrlFailure(values[4]);
                    break;
                default:
            }

        }
        @Override
        protected void onPostExecute(ScraperReport result){
            showResultsActivity(result);
        }

        @Override
        public void onCurrentUrlChanged(String currentUrl) { // 1
            this.currentUrl = currentUrl;
            publishProgress(currentUrl);
        }
        @Override
        public void onFinishedScanningUrl(String currentUrl, int scannedLinks) { // 2
            linksScanned = scannedLinks;
            publishProgress(null,currentUrl);
        }

        @Override
        public void onUrlError(String currentUrl, String message) { // 5
            publishProgress(null,null,null,null,currentUrl+" (ERROR: "+message+")");
        }

        @Override
        public void onNewLinkFound(String href, int totalLinks) { // 3
            linksPending = totalLinks;
            publishProgress(null,currentUrl,href);
        }

        @Override
        public void onWaitStatusChanged(boolean b) { // 4
            if(b) {
                publishProgress(null, null, null, getString(R.string.waiting_status_label));
                publishProgress("");
            }
            else
                publishProgress(null,null,null,getString(R.string.crawling_status_label));
        }

        @Override
        public void onScanFinished() {

        }

        /**
         * Calculates the percentile progress of the scan
         * @return The percentile progress of the scan (0 - 100)
         */
        private int getProgress(){
            int total = linksPending+linksScanned;
            if(total >0) {
                return linksScanned * 100 / total;
            }else
                return 0;
        }

        /**
         * Generates a string explaining the progress of the scan
         * @return Verbose explanation of the scan progress
         */
        private String getProgressString(){
            int progress = getProgress();
            int total = linksPending+linksScanned;
            if(total >0) {
                return progress + "% (" + linksScanned + "/" + total + ")";
            }else{
                return "";
            }
        }
    }

    /**
     * Alerts of the failure to read an url.
     * @param value Error message
     */
    private void alertUrlFailure(String value) {
        Toast.makeText(this, value, Toast.LENGTH_LONG).show();
    }

    /**
     * Updates the wait status textview
     * @param value Current status
     */
    private void updateWaitingStatus(String value) {
        this.txtStatus.setText(value);
    }

    /**
     * Updates last scanned textview
     * @param value
     */
    private void updateLastScanned(String value) {
        this.txtLastScanned.setText(getString(R.string.last_scanned_url_label)+value);
    }

    /**
     * Updates status fields
     * @param s Status of the crawling task progress
     */
    private void updateProgress(String s, int progress) {
        this.txtProgress.setText(getString(R.string.progress_label)+s);
        this.progressBar.setProgress(progress);

    }

    /**
     * Updates currentUrl Field
     * @param url Url to add to the field
     */
    private void updateCurrentUrl(String url){
        this.txtCurrentUrl.setText(getString(R.string.current_url_label)+url);
    }

    /**
     * Shows the result in the default Result Activity
     * @param result
     */
    private void showResultsActivity(ScraperReport result) {
        Intent intent = new Intent(this, DroidScrapeResultActivity.class);
        intent.putExtra(getString(R.string.result_object_name), result);
        startActivity(intent);
    }
    /**
     * Returns to the main menu
     */
    private void returnHome() {
        Intent homeIntent = new Intent(this, MainMenuActivity.class);
        homeIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivity(homeIntent);
    }

}
