/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape.plugins.webscraper.scraper.model;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * Created by Mario on 29/04/2017.
 * POJO that represents an ocurrence found in a site
 */
public class Occurrence implements Parcelable {
    private String term;
    private String link;
    private String context;

    public Occurrence(String term, String link, String context) {
        this.term = term;
        this.link = link;
        this.context = context;
    }

    protected Occurrence(Parcel in) {
        readFromParcel(in);
    }

    public static final Creator<Occurrence> CREATOR = new Creator<Occurrence>() {
        @Override
        public Occurrence createFromParcel(Parcel in) {
            return new Occurrence(in);
        }

        @Override
        public Occurrence[] newArray(int size) {
            return new Occurrence[size];
        }
    };

    public String getTerm() {
        return term;
    }

    public void setTerm(String term) {
        this.term = term;
    }

    public String getLink() {
        return link;
    }

    public void setLink(String link) {
        this.link = link;
    }

    public String getContext() {
        return context;
    }

    public void setContext(String context) {
        this.context = context;
    }

    /**
     * Generates a HTML table row with the information of the occurrence
     * @return
     */
    public String toHTMLTableRow() {
        return "<tr><td><a href=\""+this.link+"\" target=\"_blank\">"+this.link+"</a></td></tr><tr><td style=\"border:solid #085c85 1px;\">Term: "+this.term+"</td></tr>"+
                "<tr><td style=\"border:solid #085c85 1px;background-color:#2b4970;color:white;\"><code>"
                +this.context.replace("<","&lt;").replace(">","&gt;")+"</code></td></tr>";
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeString(term);
        dest.writeString(link);
        dest.writeString(context);
    }

    private void readFromParcel(Parcel in) {
        term = in.readString();
        link = in.readString();
        context = in.readString();
    }
}
