/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape.plugins.webscraper.scraper.model;

import android.os.Parcel;
import android.os.Parcelable;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import mdbdev.es.droidscrape.model.DroidScrapeResultsReport;
import mdbdev.es.droidscrape.plugins.webscraper.scraper.ScraperStatusListener;

/**
 * Interface that should be implemented by reporting classes that will show their results on a
 * standard DroidScrapeResultActivity
 */
public class ScraperReport implements DroidScrapeResultsReport {
    private String baseUrl;
    private long depth;
    private String HTMLReport;
    private List<Occurrence> occurrences;
    private List<String> searchTerms;

    public ScraperReport() {
        occurrences = new ArrayList<>();
        searchTerms = new ArrayList<>();
    }

    public ScraperReport(Parcel in){
        occurrences = new ArrayList<>();
        searchTerms = new ArrayList<>();
        readFromParcel(in);
    }

    public String getBaseUrl() {
        return baseUrl;
    }

    public void setBaseUrl(String baseUrl) {
        this.baseUrl = baseUrl;
    }

    public long getDepth() {
        return depth;
    }

    public void setDepth(long depth) {
        this.depth = depth;
    }

    public List<Occurrence> getOccurrences() {
        return occurrences;
    }

    public void setOccurrences(List<Occurrence> occurrences) {
        this.occurrences = occurrences;
    }

    public List<String> getSearchTerms() {
        return searchTerms;
    }

    public void setSearchTerms(List<String> searchTerms) {
        this.searchTerms = searchTerms;
    }

    /**
     * Generates a HTML report with all the information gathered during the scanning process
     * @return
     */
    @Override
    public String getHTMLReport(){
        String html = "<html><body><h1>Web Scraper Report</h1>";
        html+="<p>Found "+occurrences.size()+" occurrence(s) of the search terms in the website.</p><table>";
        for(Occurrence o : occurrences)
            html += o.toHTMLTableRow();
        html+="</table></body></html>";
        occurrences = null;
        return html;
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeString(baseUrl);
        dest.writeLong(depth);
        dest.writeTypedList(occurrences);
        dest.writeStringList(searchTerms);
    }

    public void readFromParcel(Parcel in){
        baseUrl = in.readString();
        depth = in.readLong();
        in.readTypedList(occurrences,Occurrence.CREATOR);
        in.readStringList(searchTerms);
    }

    public static final Parcelable.Creator<ScraperReport> CREATOR
            = new Parcelable.Creator<ScraperReport>() {
        public ScraperReport createFromParcel(Parcel in) {
            return new ScraperReport(in);
        }

        public ScraperReport[] newArray(int size) {
            return new ScraperReport[size];
        }
    };
}
