/*
 *     DroidScrape
 *     Copyright (C) 2017  Mario de Benito
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 *
 *     DroidScrape  Copyright (C) 2017  Mario de Benito
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 *
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <http://www.gnu.org/licenses/>.
 *
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <http://www.gnu.org/philosophy/why-not-lgpl.html>.
 */

package mdbdev.es.droidscrape;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.junit.Test;

import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;

import javax.mail.MessagingException;

import mdbdev.es.droidscrape.plugins.emailheaderanalyzer.analyzer.EmailHeaderAnalysisListener;
import mdbdev.es.droidscrape.plugins.emailheaderanalyzer.analyzer.EmailHeaderAnalyzer;
import mdbdev.es.droidscrape.plugins.emailheaderanalyzer.analyzer.model.AnalyzedHeader;
import mdbdev.es.droidscrape.plugins.emailheaderanalyzer.analyzer.model.EmailHeadersReport;
import mdbdev.es.droidscrape.plugins.webscraper.scraper.WebScraper;
import mdbdev.es.droidscrape.plugins.webscraper.scraper.model.ScraperMode;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;

/**
 * Created by Mario on 08/05/2017.
 * Unit testing class
 */
public class UnitTest {
    int headerPublishingCount;
    @Test
    public void EmailHeaderAnalyzerTest(){
        headerPublishingCount = 0;
        Method method = null;
        EmailHeaderAnalyzer analyzer = new EmailHeaderAnalyzer();

        try {
            method = EmailHeaderAnalyzer.class.getDeclaredMethod("checkIPAddresses", String.class);
            method.setAccessible(true);
            String result = (String) method.invoke(analyzer, "we are [192.168.1.1] testing stuff");
            assertEquals("Error analysing local IP",true,result.indexOf("Private")!=-1);
            result = (String) method.invoke(analyzer, "we are [8.8.8.8] testing stuff");
            assertEquals("Error analysing remote IP",true,result.indexOf("Mountain View")!=-1);

            method = EmailHeaderAnalyzer.class.getDeclaredMethod("checkDomains", String.class);
            method.setAccessible(true);
            result = (String) method.invoke(analyzer, "we are [ip-api.com] testing (mdb-dev.es) stuff");
            assertEquals("Error analysing domain",true,result.indexOf("Choopa")!=-1&&result.indexOf("One.com")!=-1);

            //Extracted from https://mediatemple.net/community/products/dv/204643950/understanding-an-email-header
            String headersString = "From: Media Temple user (mt.kb.user@gmail.com)\n" +
                    "Subject: article: How to Trace a Email\n" +
                    "Date: January 25, 2011 3:30:58 PM PDT\n" +
                    "To: user@example.com\n" +
                    "Return-Path: <mt.kb.user@gmail.com>\n" +
                    "Envelope-To: user@example.com\n" +
                    "Delivery-Date: Tue, 25 Jan 2011 15:31:01 -0700\n" +
                    "Received: from po-out-1718.google.com ([72.14.252.155]:54907) by cl35.gs01.gridserver.com with esmtp (Exim 4.63) (envelope-from <mt.kb.user@gmail.com>) id 1KDoNH-0000f0-RL for user@example.com; Tue, 25 Jan 2011 15:31:01 -0700\n" +
                    "Received: by po-out-1718.google.com with SMTP id y22so795146pof.4 for <user@example.com>; Tue, 25 Jan 2011 15:30:58 -0700 (PDT)\n" +
                    "Received: by 10.141.116.17 with SMTP id t17mr3929916rvm.251.1214951458741; Tue, 25 Jan 2011 15:30:58 -0700 (PDT)\n" +
                    "Received: by 10.140.188.3 with HTTP; Tue, 25 Jan 2011 15:30:58 -0700 (PDT)\n" +
                    "Dkim-Signature: v=1; a=rsa-sha256; c=relaxed/relaxed; d=gmail.com; s=gamma; h=domainkey-signature:received:received:message-id:date:from:to :subject:mime-version:content-type; bh=+JqkmVt+sHDFIGX5jKp3oP18LQf10VQjAmZAKl1lspY=; b=F87jySDZnMayyitVxLdHcQNL073DytKRyrRh84GNsI24IRNakn0oOfrC2luliNvdea LGTk3adIrzt+N96GyMseWz8T9xE6O/sAI16db48q4Iqkd7uOiDvFsvS3CUQlNhybNw8m CH/o8eELTN0zbSbn5Trp0dkRYXhMX8FTAwrH0=\n" +
                    "Domainkey-Signature: a=rsa-sha1; c=nofws; d=gmail.com; s=gamma; h=message-id:date:from:to:subject:mime-version:content-type; b=wkbBj0M8NCUlboI6idKooejg0sL2ms7fDPe1tHUkR9Ht0qr5lAJX4q9PMVJeyjWalH 36n4qGLtC2euBJY070bVra8IBB9FeDEW9C35BC1vuPT5XyucCm0hulbE86+uiUTXCkaB 6ykquzQGCer7xPAcMJqVfXDkHo3H61HM9oCQM=\n" +
                    "Message-Id: <c8f49cec0807011530k11196ad4p7cb4b9420f2ae752@mail.gmail.com>\n" +
                    "Mime-Version: 1.0\n" +
                    "Content-Type: multipart/alternative; boundary=\"----=_Part_3927_12044027.1214951458678\"\n" +
                    "X-Spam-Status: score=3.7 tests=DNS_FROM_RFC_POST, HTML_00_10, HTML_MESSAGE, HTML_SHORT_LENGTH version=3.1.7\n" +
                    "X-Spam-Level: ***\n" +
                    "Message Body: This is a KnowledgeBase article that provides information on how to find email headers and use the data to trace a email.";

            analyzer.addListener(new EmailHeaderAnalysisListener() {
                @Override
                public void updateProgress(String s) {
                    headerPublishingCount++;
                }
            });
            EmailHeadersReport report = analyzer.analyze(headersString);

            assertEquals("Error in number of headers analysed",19,report.getHeaders().size());
            assertEquals("Error in number of progress updates",19,headerPublishingCount);

            for(AnalyzedHeader ah : report.getHeaders()){
                assertNotEquals("Header ("+ah.getName()+") analysis is null",null,ah.getAnalysis());
                assertNotEquals("Header ("+ah.getName()+") analysis is empty","",ah.getAnalysis());
            }

        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (MessagingException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }


    }

    @Test
    public void WebScraperTest(){
        Method method = null;
        try {
            WebScraper ws = new WebScraper("http://mock-url.com","test1"+System.getProperty("line.separator")+"test2",0, ScraperMode.AGGRESSIVE);
            Document doc = Jsoup.parse("<html><body><span>test1</span><p>test2</p></body></html>");

            method = WebScraper.class.getDeclaredMethod("scanForOccurrences", Document.class, String.class);
            method.setAccessible(true);
            int numOccurrences = ws.getResults().size();
            String result = (String) method.invoke(ws,doc,"");
            assertEquals("Not all the occurrences were added",numOccurrences+2,ws.getResults().size());

            method = WebScraper.class.getDeclaredMethod("addAllLinks", Document.class, int.class);
            method.setAccessible(true);
            doc = Jsoup.parse("<html><body><a href=\"testlink1\">testlink1</a><span>test1</span><p>test2</p><a href=\"http://testlink2.com\">testlink2</a></body></html>");
            doc.setBaseUri("http://mock-url.com");
            int numLinksFound = 2;
            method.invoke(ws,doc,1);
            assertEquals("Not all the occurrences were added",numLinksFound,ws.getPendingLinks().size());



        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }


    }
}
